using Gtk;

class EleganceColorsWindow : ApplicationWindow {

	private Separator separator1;
	private Separator separator2;

	private Label heading1;
	private Label heading2;

	private Label description;

	private Label background_label;
	private Label arrow_label;
	private Label light_label;
	private Label icon_label;
	private Label font_label;
	private Label panel_label;
	private Label menu_label;
	private Label corner_label;

	private SpinButton font_size;
	private SpinButton panel_opacity;
	private SpinButton menu_opacity;
	private SpinButton corner_roundness;

	private Switch background_switch;
	private Switch arrow_switch;
	private Switch light_switch;
	private Switch icon_switch;

	private RadioButton match_wallpaper;
	private RadioButton match_theme;
	private RadioButton custom_color;

	private ColorButton color_button;

	private Button apply_button;
	private Button reset_button;
	private Button close_button;

	private File config_dir;
	private File config_file;

	private Gdk.RGBA color;

	private string color_value;

	private string font;
	private string panel;
	private string menu;
	private string corner;
	private string background;
	private string arrow;
	private string light;
	private string appicon;
	private string mode;

	internal EleganceColorsWindow (EleganceColorsApp app) {
		Object (application: app, title: "Elegance Colors Preferences");

		// Set window properties
		this.window_position = WindowPosition.CENTER;
		this.resizable = false;
		this.border_width = 10;

		// Set window icon
		try {
			this.icon = IconTheme.get_default ().load_icon ("preferences-desktop-theme", 48, 0);
		} catch (Error e) {
			stderr.printf ("Could not load application icon: %s\n", e.message);
		}

		// Prefer dark theme
		Gtk.Settings.get_default ().gtk_application_prefer_dark_theme = true;

		// Methods
		create_widgets ();
		connect_signals ();
	}

	private void set_defaults () {

		// Set default config
		apply_button.set_sensitive (false);
		background_switch.set_active (true);
		arrow_switch.set_active (true);
		light_switch.set_active (false);
		icon_switch.set_active (false);
		match_wallpaper.set_active (true);
		match_theme.set_active (false);
		custom_color.set_active (false);
		font_size.adjustment.value = 9;
		panel_opacity.adjustment.value = 0.8;
		menu_opacity.adjustment.value = 0.8;
		corner_roundness.adjustment.value = 6;
		color_value = "#398ee7";
	}

	private void set_config () {

		set_defaults();

		// Set the path of config file
		config_dir = File.new_for_path (Environment.get_user_config_dir ());
		config_file = config_dir.get_child ("elegance-colors.ini");

		// Read the config file
		if (config_file.query_exists ()) {
			try {
				var dis = new DataInputStream (config_file.read ());
				string line;
				while ((line = dis.read_line (null)) != null) {
					if ("font" in line) {
						font_size.adjustment.value = double.parse(line.substring (5, line.length-5));
  					}
					if ("panel" in line) {
						panel_opacity.adjustment.value = double.parse(line.substring (6, line.length-6));
  					}
					if ("menu" in line) {
						menu_opacity.adjustment.value = double.parse(line.substring (5, line.length-5));
  					}
					if ("corner" in line) {
						corner_roundness.adjustment.value = double.parse(line.substring (7, line.length-7));
  					}
					if (line == "background=on") {
						background_switch.set_active (true);
					}
					if (line == "arrow=on") {
						arrow_switch.set_active (true);
					}
					if (line == "light=on") {
						light_switch.set_active (true);
					}
					if (line == "appicon=on") {
						icon_switch.set_active (true);
					}
					if (line == "mode=wallpaper") {
						match_wallpaper.set_active (true);
						match_theme.set_active (false);
						custom_color.set_active (false);
					} else if (line == "mode=gtk") {
						match_theme.set_active (true);
						match_wallpaper.set_active (false);
						custom_color.set_active (false);
					} else if ("mode=#" in line) {
						color_value = line.substring (5, line.length-5);
						custom_color.set_active (true);
						match_theme.set_active (false);
						match_wallpaper.set_active (false);
					}
				}
			} catch (Error e) {
				stderr.printf ("%s", e.message);
			}
		}

		color = Gdk.RGBA ();
		color.parse ("%s".printf (color_value.to_string()));
		color_button.set_rgba (color);
	}

	private void create_widgets () {

		// Create and setup widgets
		separator1 = new Separator (Gtk.Orientation.HORIZONTAL);
		separator2 = new Separator (Gtk.Orientation.HORIZONTAL);

		heading1 = new Label.with_mnemonic ("<b>Settings</b>");
		heading1.set_use_markup (true);
		heading1.set_halign (Gtk.Align.START);
		heading2 = new Label.with_mnemonic ("<b>Colors</b>");
		heading2.set_use_markup (true);
		heading2.set_halign (Gtk.Align.START);

		description = new Label.with_mnemonic ("Customize various settings for Elegance Colors Gnome Shell theme.");
		description.set_halign (Gtk.Align.START);

		background_label = new Label.with_mnemonic ("Background process");
		background_label.set_halign (Gtk.Align.START);
		arrow_label = new Label.with_mnemonic ("Arrow in menu");
		arrow_label.set_halign (Gtk.Align.START);
		light_label = new Label.with_mnemonic ("Light menu style");
		light_label.set_halign (Gtk.Align.START);
		icon_label = new Label.with_mnemonic ("App icon in panel");
		icon_label.set_halign (Gtk.Align.START);
		font_label = new Label.with_mnemonic ("Font size");
		font_label.set_halign (Gtk.Align.START);
		panel_label = new Label.with_mnemonic ("Panel opacity");
		panel_label.set_halign (Gtk.Align.START);
		menu_label = new Label.with_mnemonic ("Menu opacity");
		menu_label.set_halign (Gtk.Align.START);
		corner_label = new Label.with_mnemonic ("Panel corner roundness");
		corner_label.set_halign (Gtk.Align.START);

		font_size = new SpinButton.with_range (6, 72, 1);
		panel_opacity = new SpinButton.with_range (0.1, 1.0, 0.1);
		menu_opacity = new SpinButton.with_range (0.1, 1.0, 0.1);
		corner_roundness = new SpinButton.with_range (0, 100, 1);

		background_switch = new Switch ();
		background_switch.set_halign (Gtk.Align.END);
		arrow_switch = new Switch ();
		arrow_switch.set_halign (Gtk.Align.END);
		light_switch = new Switch ();
		light_switch.set_halign (Gtk.Align.END);
		icon_switch = new Switch ();
		icon_switch.set_halign (Gtk.Align.END);

		match_wallpaper = new RadioButton (null);
		match_wallpaper.set_label ("Match wallpaper color");
		match_theme = new RadioButton.with_label (match_wallpaper.get_group(),"Match GTK theme color");
		custom_color = new RadioButton.with_label (match_theme.get_group(),"Use custom color");

		color_button = new ColorButton ();

		apply_button = new Button.from_stock (Stock.APPLY);
		reset_button = new Button.from_stock(Stock.REVERT_TO_SAVED);
		close_button = new Button.from_stock (Stock.CLOSE);

		// Layout widgets
		var grid = new Grid ();
		grid.set_column_homogeneous (true);
		grid.set_column_spacing (10);
		grid.set_row_spacing (10);
		grid.set_border_width (10);
		grid.attach (heading1, 0, 0, 1, 1);
		grid.attach_next_to (separator1, heading1, Gtk.PositionType.RIGHT, 2, 1);
		grid.attach (description, 0, 1, 3, 1);
		grid.attach (background_label, 0, 2, 2, 1);
		grid.attach_next_to (background_switch, background_label, Gtk.PositionType.RIGHT, 1, 1);
		grid.attach (arrow_label, 0, 3, 2, 1);
		grid.attach_next_to (arrow_switch, arrow_label, Gtk.PositionType.RIGHT, 1, 1);
		grid.attach (light_label, 0, 4, 2, 1);
		grid.attach_next_to (light_switch, light_label, Gtk.PositionType.RIGHT, 1, 1);
		grid.attach (icon_label, 0, 5, 2, 1);
		grid.attach_next_to (icon_switch, icon_label, Gtk.PositionType.RIGHT, 1, 1);
		grid.attach (font_label, 0, 6, 2, 1);
		grid.attach_next_to (font_size, font_label, Gtk.PositionType.RIGHT, 1, 1);
		grid.attach (panel_label, 0, 7, 2, 1);
		grid.attach_next_to (panel_opacity, panel_label, Gtk.PositionType.RIGHT, 1, 1);
		grid.attach (menu_label, 0, 8, 2, 1);
		grid.attach_next_to (menu_opacity, menu_label, Gtk.PositionType.RIGHT, 1, 1);
		grid.attach (corner_label, 0, 9, 2, 1);
		grid.attach_next_to (corner_roundness, corner_label, Gtk.PositionType.RIGHT, 1, 1);
		grid.attach (heading2, 0, 10, 1, 1);
		grid.attach_next_to (separator2, heading2, Gtk.PositionType.RIGHT, 2, 1);
		grid.attach (match_wallpaper, 0, 11, 2, 1);
		grid.attach (match_theme, 0, 12, 2, 1);
		grid.attach (custom_color, 0, 13, 2, 1);
		grid.attach_next_to (color_button, custom_color, Gtk.PositionType.RIGHT, 1, 1);
		grid.attach (apply_button, 0, 14, 1, 1);
		grid.attach_next_to (reset_button, apply_button, Gtk.PositionType.RIGHT, 1, 1);
		grid.attach_next_to (close_button, reset_button, Gtk.PositionType.RIGHT, 1, 1);

		this.add (grid);

		set_config ();
	}

	private void connect_signals () {
		font_size.adjustment.value_changed.connect (() => {
			apply_button.set_sensitive (true);
		});
		panel_opacity.adjustment.value_changed.connect (() => {
			apply_button.set_sensitive (true);
		});
		menu_opacity.adjustment.value_changed.connect (() => {
			apply_button.set_sensitive (true);
		});
		corner_roundness.adjustment.value_changed.connect (() => {
			apply_button.set_sensitive (true);
		});
		background_switch.notify["active"].connect (() => {
			apply_button.set_sensitive (true);
		});
		arrow_switch.notify["active"].connect (() => {
			apply_button.set_sensitive (true);
		});
		light_switch.notify["active"].connect (() => {
			apply_button.set_sensitive (true);
		});
		icon_switch.notify["active"].connect (() => {
			apply_button.set_sensitive (true);
		});
		match_wallpaper.toggled.connect (() => {
			apply_button.set_sensitive (true);
		});
		match_theme.toggled.connect (() => {
			apply_button.set_sensitive (true);
		});
		custom_color.toggled.connect (() => {
			apply_button.set_sensitive (true);
		});
		color_button.color_set.connect (() => {
			on_selected_color_set ();
			custom_color.set_active (true);
			apply_button.set_sensitive (true);
		});
		apply_button.clicked.connect (() => {
			write_config ();
			apply_button.set_sensitive (false);
			reset_button.set_sensitive (true);
		});
		reset_button.clicked.connect (() => {
			set_defaults ();
			write_config ();
			reset_button.set_sensitive (false);
		});
		close_button.clicked.connect (() => {
			destroy ();
		});
	}

	private void on_selected_color_set () {
		color =  color_button.get_rgba ();

		int r = (int)Math.round (color.red * 255);
		int g = (int)Math.round (color.green * 255);
		int b = (int)Math.round (color.blue * 255);

		color_value = "#%02x%02x%02x".printf (r, g, b);
	}

	private void write_config () {
		font = "font=%g".printf(font_size.adjustment.value);
		panel = "panel=%g".printf(panel_opacity.adjustment.value);
		menu = "menu=%g".printf(menu_opacity.adjustment.value);
		corner = "corner=%g".printf(corner_roundness.value);
		if (background_switch.get_active()) {
			background = "background=on";
		} else {
			background = "background=off";
		}
		if (arrow_switch.get_active()) {
			arrow = "arrow=on";
		} else {
			arrow = "arrow=off";
		}
		if (light_switch.get_active()) {
			light = "light=on";
		} else {
			light = "light=off";
		}
		if (icon_switch.get_active()) {
			appicon = "appicon=on";
		} else {
			appicon = "appicon=off";
		}
		if (match_wallpaper.get_active()) {
			mode = "mode=wallpaper";
		} else if (match_theme.get_active()) {
			mode = "mode=gtk";
		} else if (custom_color.get_active()) {
			mode = "mode=%s".printf(color_value);
		} else {
			mode = "mode=wallpaper";
		}
		if (config_file.query_exists ()) {
			try {
				config_file.delete ();
			} catch (Error e) {
				stderr.printf ("%s", e.message);
			}
		}
		try {
			var dos = new DataOutputStream (config_file.create (FileCreateFlags.REPLACE_DESTINATION));
			dos.put_string ("[elegance-colors]\n");
			string text = "%s\n%s\n%s\n%s\n%s\n%s\n%s\n%s\n%s".printf(font, panel, menu, corner, background, arrow, light, appicon, mode);
			uint8[] data = text.data;
			long written = 0;
			while (written < data.length) {
				written += dos.write (data[written:data.length]);
			}
		} catch (Error e) {
			stderr.printf ("%s", e.message);
		}
		try {
			Process.spawn_command_line_sync("notify-send -i \"preferences-desktop-theme\" \"Changes will be applied shortly\"");
		} catch (Error e) {
			stderr.printf ("%s", e.message);
		}
	}
}

class EleganceColorsApp : Gtk.Application {
	protected override void activate () {

		var window = new EleganceColorsWindow (this);
		window.show_all ();
	}

	internal EleganceColorsApp () {
		Object (application_id: "org.elegance.app");
	}
}

int main (string[] args) {
	return new EleganceColorsApp ().run (args);
}

